package com.myco.jmstest;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;

import javax.jms.Message;
import javax.jms.MessageListener;

import org.apache.log4j.Logger;
import org.springframework.jms.support.converter.MessageConversionException;

public class JmsListener implements MessageListener {

    private static Logger LOGGER = Logger.getLogger(JmsListener.class);
    
    private long lastPrintedTime = 0;
    private AtomicBoolean currentlyPrinting = new AtomicBoolean(false);
    private AtomicInteger messagesReceived = new AtomicInteger(0);
    private AtomicInteger totalReceived = new AtomicInteger(0);
    
    @Override
    public void onMessage(Message msg) {
        try {
            messagesReceived.incrementAndGet();
            totalReceived.incrementAndGet();
            if(System.currentTimeMillis() - lastPrintedTime > 5000) {
                if(currentlyPrinting.compareAndSet(false, true)) {
                    if(System.currentTimeMillis() - lastPrintedTime > 5000) {
                        LOGGER.info("Messages received: " + messagesReceived.getAndSet(0));
                        lastPrintedTime = System.currentTimeMillis();
                    }
                    currentlyPrinting.set(false);
                }
            }
        } catch (MessageConversionException e) {
            LOGGER.error("Unexpected MessageConversionException: ", e);
        }
    }

    public int getNumberOfReceivedMsgs() {
        return totalReceived.get();
    }
    
    public void resetNumberOfReceivedMsgs() {
        totalReceived.set(0);
        messagesReceived.set(0);
    }
}
