package com.myco.jmstest;

import java.util.ArrayList;
import java.util.List;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.apache.log4j.Logger;

public class JmsSenderWithoutJmsTemplate extends AbstractJmsSender {

    private static final Logger LOGGER = Logger.getLogger(JmsSenderWithoutJmsTemplate.class);
    
    private final List<SessionAndProducer> sessionsAndProducers = new ArrayList<SessionAndProducer>();
    private final Object lock = new Object();
    
    private int nextSessionAndProducer = 0;
    
    public JmsSenderWithoutJmsTemplate(Queue queue, ConnectionFactory connectionFactory, int numThreads) {
        super(numThreads);
        
        try {
            Connection connection = connectionFactory.createConnection();
            
            for(int i = 0; i < numThreads; i++) {
                Session session = connection.createSession(false,  Session.CLIENT_ACKNOWLEDGE);
                MessageProducer producer = session.createProducer(queue);
                sessionsAndProducers.add(new SessionAndProducer(session, producer));
            }
            
        } catch (JMSException e) {
            e.printStackTrace();
        }
    }

    private class SessionAndProducer {
        
        public final Session session;
        public final MessageProducer producer;
        
        public SessionAndProducer(Session session, MessageProducer producer) {
            this.session = session;
            this.producer = producer;
        }
    }
    
    private SessionAndProducer getNextSessionAndProducer() {
        SessionAndProducer sessuionAndProducer = null;
        synchronized (lock) {
            if(sessionsAndProducers.size() > 0) {
                int index = nextSessionAndProducer;
                if(nextSessionAndProducer + 1 < sessionsAndProducers.size()) {
                    nextSessionAndProducer++;
                } else {
                    nextSessionAndProducer = 0;
                }
                sessuionAndProducer = sessionsAndProducers.get(index);
            }
        }
        return sessuionAndProducer;
    }
    
    /* (non-Javadoc)
     * @see com.myco.jmstest.AbstractJmsSender#send(java.lang.String)
     */
    @Override
    protected void send(String msg) {
        try {
            SessionAndProducer sessionAndProducer = getNextSessionAndProducer();
            TextMessage textMsg = sessionAndProducer.session.createTextMessage(msg);
            sessionAndProducer.producer.send(textMsg);
        } catch (JMSException e) {
            LOGGER.error("Unable able to send message", e);
        }
    }
}
